class OrderTotalCalculator(object):
    """
    Calculator class for calculating the order total.
    """
    
    def __init__(self, request):
        # We store a reference to the request as the total may 
        # depend on the user or the other checkout data in the session.
        # Further, it is very likely that it will as shipping method
        # always changes the order total.
        self.request = request
    
    def order_total_incl_tax(self, basket, shipping_method=None, **kwargs):
        """
        Return order total including tax
        """
        # Default to returning the total including tax - use
        # the request.user object if you want to not charge tax
        # to particular customers.  
        total = basket.total_incl_tax
        if shipping_method:
            total += shipping_method.basket_charge_incl_tax()
        return total
    
    def order_total_excl_tax(self, basket, shipping_method=None, **kwargs):
        """
        Return order total excluding tax
        """
        total = basket.total_excl_tax
        if shipping_method:
            total += shipping_method.basket_charge_excl_tax()
        return total


from django.contrib import admin

from oscar.core.loading import import_module
product_models = import_module('catalogue.models', ['Product', 'ProductClass', 'ProductAttribute', 'ProductAttributeValue', 
                                                    'Option', 'ProductRecommendation', 'AttributeOption', 'AttributeOptionGroup',
                                                    'AttributeEntity', 'AttributeEntityType', 'ProductImage', 'Category', 
                                                    'ProductCategory'])

class AttributeInline(admin.TabularInline):
    model = product_models.ProductAttributeValue

class ProductRecommendationInline(admin.TabularInline):
    model = product_models.ProductRecommendation
    fk_name = 'primary'
    
class CategoryInline(admin.TabularInline):
    model = product_models.ProductCategory
    extra = 1

class ProductClassAdmin(admin.ModelAdmin):
    prepopulated_fields = {"slug": ("name",)}
    
class ProductAdmin(admin.ModelAdmin):
    list_display = ('get_title', 'upc', 'get_product_class', 'is_top_level', 'is_group', 'is_variant', 'attribute_summary', 'date_created')
    prepopulated_fields = {"slug": ("title",)}
    inlines = [AttributeInline, CategoryInline, ProductRecommendationInline]
    
class ProductAttributeAdmin(admin.ModelAdmin):
    prepopulated_fields = {"code": ("name", )}
    
class OptionAdmin(admin.ModelAdmin):
    exclude = ['code']
    
class ProductAttributeValueAdmin(admin.ModelAdmin):
    list_display = ('product', 'attribute', 'value')

class AttributeOptionInline(admin.TabularInline):
    model = product_models.AttributeOption
    
class AttributeOptionGroupAdmin(admin.ModelAdmin):
    inlines = [AttributeOptionInline, ]
        
class AttributeEntityAdmin(admin.ModelAdmin):
    list_display = ('name', )
                 

admin.site.register(product_models.ProductClass, ProductClassAdmin)
admin.site.register(product_models.Product, ProductAdmin)
admin.site.register(product_models.ProductAttribute, ProductAttributeAdmin)
admin.site.register(product_models.ProductAttributeValue, ProductAttributeValueAdmin)
admin.site.register(product_models.AttributeOptionGroup, AttributeOptionGroupAdmin)
admin.site.register(product_models.AttributeEntity, AttributeEntityAdmin)
admin.site.register(product_models.AttributeEntityType)
admin.site.register(product_models.Option, OptionAdmin)
admin.site.register(product_models.ProductImage)
admin.site.register(product_models.Category)



from django.conf.urls.defaults import *
from django.contrib import admin
from django.views.generic.simple import redirect_to

from django.conf import settings
from oscar.app import Shop
from oscar.apps.catalogue import ProductApplication
from shop.product.views import MyItemDetailView

admin.autodiscover()

shop_app = Shop(product_app=ProductApplication(detail_view=MyItemDetailView))

urlpatterns = patterns('',
    (r'^admin/', include(admin.site.urls)),
    (r'', include(shop_app.urls)),
)


if settings.DEBUG:
    urlpatterns += patterns('django.views.static',
        url(r'^media/(?P<path>.*)$', 'serve', 
            {'document_root': settings.MEDIA_ROOT}),
    )




class Bankcard(object):
    
    def __init__(self, card_number, expiry_date, name=None, ccv=None, start_date=None, issue_number=None):
        self.card_number = card_number
        self.card_holder_name = name
        self.expiry_date = expiry_date
        self.start_date = start_date
        self.issue_number = issue_number
        self.ccv = ccv


from decimal import Decimal

from django.db import models
from django.utils.translation import ugettext as _


class AbstractProductRecord(models.Model):
    u"""
    A record of a how popular a product is.
    
    This used be auto-merchandising to display the most popular
    products.
    """
    
    product = models.OneToOneField('catalogue.Product')
    
    # Data used for generating a score
    num_views = models.PositiveIntegerField(default=0)
    num_basket_additions = models.PositiveIntegerField(default=0)
    num_purchases = models.PositiveIntegerField(default=0, db_index=True)
    
    # Product score - used within search
    score = models.FloatField(default=0.00)
    
    class Meta:
        abstract = True
        ordering = ['-num_purchases']
        
    def __unicode__(self):
        return u"Record for '%s'" % self.product
        

class AbstractUserRecord(models.Model):
    u"""
    A record of a user's activity.
    """
    
    user = models.OneToOneField('auth.User')
    
    # Browsing stats
    num_product_views = models.PositiveIntegerField(default=0)
    num_basket_additions = models.PositiveIntegerField(default=0)
    
    # Order stats
    num_orders = models.PositiveIntegerField(default=0, db_index=True)
    num_order_lines = models.PositiveIntegerField(default=0, db_index=True)
    num_order_items = models.PositiveIntegerField(default=0, db_index=True)
    total_spent = models.DecimalField(decimal_places=2, max_digits=12, default=Decimal('0.00'))
    date_last_order = models.DateTimeField(blank=True, null=True)
    
    class Meta:
        abstract = True
        
        
class AbstractUserProductView(models.Model):
    
    user = models.ForeignKey('auth.User')
    product = models.ForeignKey('catalogue.Product')
    date_created = models.DateTimeField(auto_now_add=True)
     
    class Meta:
        abstract = True
        
    def __unicode__(self):
        return u"%s viewed '%s'" % (self.user, self.product)
             

class AbstractUserSearch(models.Model):
    
    user = models.ForeignKey('auth.User')
    query = models.CharField(_("Search term"), max_length=255, db_index=True)
    date_created = models.DateTimeField(auto_now_add=True)
     
    class Meta:
        abstract = True
        verbose_name_plural = _("User search queries")
        
    def __unicode__(self):
        return u"%s searched for '%s'" % (self.user, self.query)



import json

from django.dispatch import receiver
from django.conf import settings

from oscar.core.loading import import_module
import_module('catalogue.signals', ['product_viewed'], locals())

MAX_PRODUCTS = settings.OSCAR_RECENTLY_VIEWED_PRODUCTS

# Helpers

def get_recently_viewed_product_ids(request):
    u"""
    Returns the list of ids of the last products browsed by the user
    
    Limited to the max number defined in settings.py
    under OSCAR_RECENTLY_VIEWED_PRODUCTS.
    """
    product_ids = [];
    if (request.COOKIES.has_key('oscar_recently_viewed_products')):
        try:
            product_ids = _get_list_from_json_string(request.COOKIES['oscar_recently_viewed_products'])
        except ValueError:
            # This can occur if something messes up the cookie
            pass
    return product_ids

def _update_recently_viewed_products(product, request, response):
    """
    Updates the cookies that store the recently viewed products
    removing possible duplicates.
    """
    product_ids = get_recently_viewed_product_ids(request)
    if product.id in product_ids:
        product_ids.remove(product.id)
    product_ids.append(product.id)
    if (len(product_ids) > MAX_PRODUCTS):
        product_ids = product_ids[len(product_ids)-MAX_PRODUCTS:]
    response.set_cookie('oscar_recently_viewed_products', 
                        _get_json_string_from_list(product_ids),
                        httponly=True)

def _get_list_from_json_string(cookie_value):
    u""" Simple function to convert lists to json """
    return json.loads(cookie_value)

def _get_json_string_from_list(list):
    """ Simple function to convert json to a python list """
    return json.dumps(list)


# Receivers

@receiver(product_viewed)
def receive_product_view(sender, product, user, request, response, **kwargs):
    """
    Receiver to handle viewing single product pages
    
    Requires the request and response objects due to dependence on cookies
    """
    return _update_recently_viewed_products(product, request, response)



from decimal import Decimal

from django.test import TestCase

from oscar.apps.address.models import Country
from oscar.apps.basket.models import Basket
from oscar.apps.order.models import ShippingAddress, Order, Line, ShippingEvent, ShippingEventType, ShippingEventQuantity

ORDER_PLACED = 'order_placed'


class ShippingAddressTest(TestCase):
    
    def test_titleless_salutation_is_stripped(self):
        country = Country.objects.get(iso_3166_1_a2='GB')
        a = ShippingAddress.objects.create(last_name='Barrington', line1="75 Smith Road", postcode="N4 8TY", country=country)
        self.assertEquals("Barrington", a.salutation())
    

class OrderTest(TestCase):
    fixtures = ['sample-order.json']

    def setUp(self):
        self.order = Order.objects.get(number='100002')
        
    def event(self, type):
        """
        Creates an order-level shipping event
        """
        type = ShippingEventType.objects.get(code=type)
        event = ShippingEvent.objects.create(order=self.order, event_type=type)
        for line in self.order.lines.all():
            ShippingEventQuantity.objects.create(event=event, line=line)  
        
    def test_shipping_status_is_empty_with_no_events(self):
        self.assertEquals("", self.order.shipping_status)

    def test_shipping_status_after_one_order_level_events(self):
        self.event(ORDER_PLACED)
        self.assertEquals("Order placed", self.order.shipping_status)

        
class LineTest(TestCase):
    fixtures = ['sample-order.json']

    def setUp(self):
        self.order = Order.objects.get(number='100002')
        self.line = self.order.lines.get(id=1)

    def event(self, type, quantity=None):
        """
        Creates a shipping event for the test line
        """
        event = ShippingEvent.objects.create(order=self.order, event_type=type)
        if quantity == None:
            quantity = self.line.quantity
        event_quantity = ShippingEventQuantity.objects.create(event=event, line=self.line, quantity=quantity)

    def test_shipping_status_is_empty_to_start_with(self):
        self.assertEquals('', self.line.shipping_status)
        
    def test_shipping_status_after_full_line_event(self):
        type = ShippingEventType.objects.get(code='order_placed')
        self.event(type)
        self.assertEquals(type.name, self.line.shipping_status)    
        
    def test_shipping_status_after_two_full_line_events(self):
        type1 = ShippingEventType.objects.get(code='order_placed')
        self.event(type1)
        type2 = ShippingEventType.objects.get(code='dispatched')
        self.event(type2)
        self.assertEquals(type2.name, self.line.shipping_status) 
        
    def test_shipping_status_after_partial_line_event(self):
        type = ShippingEventType.objects.get(code='order_placed')
        self.event(type, 3)
        expected = "%s (%d/%d items)" % (type.name, 3, self.line.quantity)
        self.assertEquals(expected, self.line.shipping_status) 
        
    def test_has_passed_shipping_status_after_full_line_event(self):
        type = ShippingEventType.objects.get(code='order_placed')
        self.event(type)
        self.assertTrue(self.line.has_shipping_event_occurred(type)) 
        
    def test_has_passed_shipping_status_after_partial_line_event(self):
        type = ShippingEventType.objects.get(code='order_placed')
        self.event(type, self.line.quantity - 1)
        self.assertFalse(self.line.has_shipping_event_occurred(type)) 
        
    def test_has_passed_shipping_status_after_multiple_line_event(self):
        event_types = [ShippingEventType.objects.get(code='order_placed'),
                        ShippingEventType.objects.get(code='dispatched')]
        for type in event_types:
            self.event(type)
        for type in event_types:
            self.assertTrue(self.line.has_shipping_event_occurred(type))
            
    def test_inconsistent_shipping_status_setting(self):
        type = ShippingEventType.objects.get(code='order_placed')
        self.event(type, self.line.quantity - 1)
        
        with self.assertRaises(ValueError):
            # Quantity is higher for second event than first
            type = ShippingEventType.objects.get(code='dispatched')
            self.event(type, self.line.quantity)
        
    def test_inconsistent_shipping_quantities(self):
        type = ShippingEventType.objects.get(code='order_placed')
        self.event(type, self.line.quantity - 1)
        
        with self.assertRaises(ValueError):
            # Total quantity is too high
            self.event(type, 2)
        
        
class ShippingEventQuantityTest(TestCase):
    fixtures = ['sample-order.json']

    def setUp(self):
        self.order = Order.objects.get(number='100002')
        self.line = self.order.lines.get(id=1)

    def test_quantity_defaults_to_all(self):
        type = ShippingEventType.objects.get(code='order_placed')
        event = ShippingEvent.objects.create(order=self.order, event_type=type)
        event_quantity = ShippingEventQuantity.objects.create(event=event, line=self.line)
        self.assertEquals(self.line.quantity, event_quantity.quantity)
    
   


from haystack import site
from haystack.exceptions import AlreadyRegistered

from oscar.apps.search.abstract_indexes import AbstractProductIndex
from oscar.core.loading import import_module
product_models = import_module('catalogue.models', ['Product'])


class ProductIndex(AbstractProductIndex):
    pass


try:
    site.register(product_models.Product, ProductIndex)
except AlreadyRegistered:
    # If already registered, it means that a different search app is being
    # used to search products.
    pass




from django.conf.urls.defaults import patterns, url, include

from oscar.core.application import Application
from oscar.apps.catalogue.views import ProductDetailView, ProductListView, ProductCategoryView
from oscar.apps.catalogue.reviews.app import application as reviews_app


class BaseCatalogueApplication(Application):
    name = 'catalogue'
    detail_view = ProductDetailView
    index_view = ProductListView
    category_view = ProductCategoryView
    
    def get_urls(self):
        urlpatterns = super(BaseCatalogueApplication, self).get_urls()        
        urlpatterns += patterns('',
            url(r'^$', self.index_view.as_view(), name='index'),       
            url(r'^(?P<product_slug>[\w-]*)-(?P<pk>\d+)/$', self.detail_view.as_view(), name='detail'),
            url(r'^(?P<category_slug>[\w-]+(/[\w-]+)*)/$', self.category_view.as_view(), name='category')
        )
        return urlpatterns


class ReviewsApplication(Application):
    reviews_app = reviews_app    
    
    def get_urls(self):
        urlpatterns = super(ReviewsApplication, self).get_urls()
        urlpatterns += patterns('',
            url(r'^(?P<product_slug>[\w-]*)-(?P<product_pk>\d+)/reviews/', include(self.reviews_app.urls)),
        )
        return urlpatterns


class CatalogueApplication(BaseCatalogueApplication, ReviewsApplication):
    """
    Composite class combining Products with Reviews
    """

application = CatalogueApplication()


from datetime import date, datetime
from calendar import monthrange
import re

from django import forms

from oscar.core.loading import import_module

address_models = import_module('address.models', ['Country'])
order_models = import_module('order.models', ['BillingAddress'])
payment_models = import_module('payment.models', ['Bankcard'])
import_module('payment.utils', ['Bankcard'], locals())

VISA, MASTERCARD, AMEX, MAESTRO, DISCOVER = ('Visa', 'Mastercard', 'American Express', 'Maestro', 'Discover')

def bankcard_type(number):
    u"""
    Returns the type of a bankcard based on its number.
    """
    number = str(number)
    if len(number) == 13:
        if number[0] == "4":
            return VISA
    elif len(number) == 14:
        if number[:2] == "36":
            return MASTERCARD
    elif len(number) == 15:
        if number[:2] in ("34", "37"):
            return AMEX
    elif len(number) == 16:
        if number[:4] == "6011":
            return DISCOVER
        if number[:2] in ("51", "52", "53", "54", "55"):
            return MASTERCARD
        if number[0] == "4":
            return VISA
    return None

def luhn(card_number):
    u"""
    Tests whether a bankcard number passes the Luhn algorithm.
    """
    card_number = str(card_number)
    sum = 0
    num_digits = len(card_number)
    odd_even = num_digits & 1

    for i in range(0, num_digits):
        digit = int(card_number[i])
        if not (( i & 1 ) ^ odd_even ):
            digit = digit * 2
        if digit > 9:
            digit = digit - 9
        sum = sum + digit

    return (sum % 10) == 0


class BankcardNumberField(forms.CharField):

    def clean(self, value):
        """Check if given CC number is valid and one of the
           card types we accept"""
        non_decimal = re.compile(r'\D+')
        value = non_decimal.sub('', value.strip())    
           
        if value and not luhn(value):
            raise forms.ValidationError("Please enter a valid credit card number.")
        return super(BankcardNumberField, self).clean(value)


class BankcardMonthWidget(forms.MultiWidget):
    """ 
    Widget containing two select boxes for selecting the month and year
    """
    def decompress(self, value):
        return [value.month, value.year] if value else [None, None]

    def format_output(self, rendered_widgets):
        html = u' '.join(rendered_widgets)
        return u'<span style="white-space: nowrap">%s</span>' % html


class BankcardMonthField(forms.MultiValueField):
    """
    A modified version of the snippet: http://djangosnippets.org/snippets/907/
    """
    default_error_messages = {
        'invalid_month': u'Enter a valid month.',
        'invalid_year': u'Enter a valid year.',
    }
    num_years = 5

    def __init__(self, *args, **kwargs):
        
        # Allow the number of years to be specified
        if 'num_years' in kwargs:
            self.num_years = kwargs['num_years']
            del kwargs['num_years']
        
        errors = self.default_error_messages.copy()
        if 'error_messages' in kwargs:
            errors.update(kwargs['error_messages'])
        fields = (
            forms.ChoiceField(choices=self.month_choices(),
                error_messages={'invalid': errors['invalid_month']}),
            forms.ChoiceField(choices=self.year_choices(),
                error_messages={'invalid': errors['invalid_year']}),
        )
        
        super(BankcardMonthField, self).__init__(fields, *args, **kwargs)
        self.widget = BankcardMonthWidget(widgets = [fields[0].widget, fields[1].widget])
        
    def month_choices(self):
        return []
    
    def year_choices(self):
        return []


class BankcardExpiryMonthField(BankcardMonthField):
    """
    Expiry month
    """
    def month_choices(self):
        return [("%.2d" % x, "%.2d" % x) for x in xrange(1, 13)]

    def year_choices(self):
        return [(x, x) for x in xrange(date.today().year, date.today().year+self.num_years)]

    def clean(self, value):
        expiry_date = super(BankcardExpiryMonthField, self).clean(value)
        if date.today() > expiry_date:
            raise forms.ValidationError("The expiration date you entered is in the past.")
        return expiry_date

    def compress(self, data_list):
        if data_list:
            if data_list[1] in forms.fields.EMPTY_VALUES:
                error = self.error_messages['invalid_year']
                raise forms.ValidationError(error)
            if data_list[0] in forms.fields.EMPTY_VALUES:
                error = self.error_messages['invalid_month']
                raise forms.ValidationError(error)
            year = int(data_list[1])
            month = int(data_list[0])
            # find last day of the month
            day = monthrange(year, month)[1]
            return date(year, month, day)
        return None 
    

class BankcardStartingMonthField(BankcardMonthField):
    """
    Starting month
    """
    def month_choices(self):
        months = [("%.2d" % x, "%.2d" % x) for x in xrange(1, 13)]
        months.insert(0, ("", "--"))
        return months

    def year_choices(self):
        years = [(x, x) for x in xrange( date.today().year - self.num_years, date.today().year)]
        years.insert(0, ("", "--"))
        return years

    def clean(self, value):
        starting_date = super(BankcardMonthField, self).clean(value)
        if starting_date and date.today() < starting_date:
            raise forms.ValidationError("The starting date you entered is in the future.")
        return starting_date

    def compress(self, data_list):
        if data_list:
            if data_list[1] in forms.fields.EMPTY_VALUES:
                error = self.error_messages['invalid_year']
                raise forms.ValidationError(error)
            if data_list[0] in forms.fields.EMPTY_VALUES:
                error = self.error_messages['invalid_month']
                raise forms.ValidationError(error)
            year = int(data_list[1])
            month = int(data_list[0])
            return date(year, month, 1)
        return None 
    

class BankcardForm(forms.ModelForm):
    
    number = BankcardNumberField(max_length=20, widget=forms.TextInput(attrs={'autocomplete':'off'}), label="Card number")
    name = forms.CharField(max_length=128, label="Name on card")
    ccv_number = forms.IntegerField(required=True, label="CCV Number",
        max_value = 99999, widget=forms.TextInput(attrs={'size': '5'}))
    start_month = BankcardStartingMonthField(label="Valid from", required=False)
    expiry_month = BankcardExpiryMonthField(required=True, label = "Valid to")
    
    class Meta:
        model = payment_models.Bankcard
        exclude = ('user', 'partner_reference')
        fields = ('number', 'name', 'start_month', 'expiry_month', 'ccv_number')
        
    def get_bankcard_obj(self):
        """
        Returns a Bankcard object for use in payment processing.
        """
        kwargs = {
            'card_number': self.cleaned_data['number'],
            'expiry_date': self.cleaned_data['expiry_month'].strftime("%m/%y"),
            'ccv': self.cleaned_data['ccv_number'],
        }
        if self.cleaned_data['start_month']:
            kwargs['start_date'] = self.cleaned_data['start_month'].strftime("%m/%y")
        return Bankcard(**kwargs)
    

class BillingAddressForm(forms.ModelForm):
    
    def __init__(self, *args, **kwargs):
        super(BillingAddressForm,self ).__init__(*args, **kwargs)
        self.set_country_queryset() 
        
    def set_country_queryset(self):    
        self.fields['country'].queryset = address_models.Country._default_manager.all()
    
    class Meta:
        model = order_models.BillingAddress
   


from django.http import HttpResponseRedirect
from django.contrib import messages
from django.core.urlresolvers import reverse

from oscar.core.loading import import_module
import_module('checkout.utils', ['ProgressChecker'], locals())


def prev_steps_must_be_complete(view_fn):
    """
    Decorator for checking that previous steps of the checkout
    are complete.
    
    The completed steps (identified by URL-names) are stored in the session.
    If this fails, then we redirect to the next uncompleted step.
    """
    def _view_wrapper(self, request, *args, **kwargs):
        checker = ProgressChecker()
        if not checker.are_previous_steps_complete(request):
            messages.error(request, "You must complete this step of the checkout first")
            url_name = checker.get_next_step(request)
            return HttpResponseRedirect(reverse(url_name))
        return view_fn(self, request, *args, **kwargs)
    return _view_wrapper


def basket_required(view_fn):
    """
    Decorator for checking that the user has a non-empty basket
    or has a frozen one in the session
    """
    def _view_wrapper(self, request, *args, **kwargs):
        if request.basket.is_empty and not 'checkout_basket_id' in request.session:
            messages.error(request, "You must add some products to your basket before checking out")
            return HttpResponseRedirect(reverse('oscar-basket'))
        return view_fn(self, request, *args, **kwargs)
    return _view_wrapper





from oscar.apps.shipping.methods import FreeShipping
from oscar.core.loading import import_module
shipping_models = import_module('shipping.models', ['OrderAndItemLevelChargeMethod'])


class Repository(object):
    u"""
    Repository class responsible for returning ShippingMethod
    objects
    """
    
    def get_shipping_methods(self, user, basket, shipping_addr):
        u"""
        Returns all applicable shipping method objects
        for a given basket.
        
        We default to returning the Method models that have been defined but
        this behaviour can easily be overridden by subclassing this class
        and overriding this method.
        """ 
        methods = shipping_models.OrderAndItemLevelChargeMethod._default_manager.all()
        if not methods.count():
            return [FreeShipping()]
        
        for method in methods:
            method.set_basket(basket)
        return methods

    def find_by_code(self, code):
        u"""
        Returns the appropriate Method object for the given code
        """
        if code == FreeShipping.code:
            return FreeShipping()
        return shipping_models.OrderAndItemLevelChargeMethod._default_manager.get(code=code)          



from random import randint
from sys import maxint

from django.test import TestCase, Client
from django.core.exceptions import ValidationError
from django.contrib.auth.models import User, AnonymousUser
from django.db import IntegrityError
from django.core.urlresolvers import reverse
from django.utils import unittest

from oscar.apps.catalogue.reviews.models import ProductReview, Vote
from oscar.test.helpers import create_product


class ProductReviewTests(unittest.TestCase):
    u"""
    Basic setup
    """
    def setUp(self):
        username = str(randint(0, maxint))
        self.user = User.objects.create_user(username, '%s@users.com'%username, '%spass123'%username)
        self.anon_user = AnonymousUser()
        self.product = create_product()
        self.review = ProductReview.objects.create(product=self.product,
                                                   title="Dummy review",
                                                   score=3,
                                                   user=self.user)

    def test_top_level_reviews_must_have_titles_and_scores(self):
        self.assertRaises(ValidationError, ProductReview.objects.create, product=self.product,
                          user=self.user)

    def test_top_level_anonymous_reviews_must_have_names_and_emails(self):
        self.assertRaises(ValidationError, ProductReview.objects.create, product=self.product,
                          user=None, title="Anonymous review", score=3)


class TopLevelProductReviewVoteTests(ProductReviewTests):
    """
    Basic tests for Vote model
    """

    def test_try_vote_without_login(self):
        self.assertRaises(ValueError, Vote.objects.create, review=self.review, delta=-1, user=self.anon_user)

    def test_try_vote_more_than_once(self):
        vote1 = Vote.objects.create(review=self.review, user=self.user, delta=1)
        self.assertTrue(vote1)
        self.assertRaises(IntegrityError, Vote.objects.create, review=self.review, delta=-1, user=self.user)


class SingleProductReviewViewTest(ProductReviewTests, TestCase):
    u"""
    Tests for each product review 
    """
    def setUp(self):
        self.client = Client()
        super(SingleProductReviewViewTest, self).setUp()
        self.kwargs = {
                'product_slug': self.product.slug,
                'pk': str(self.product.id)}
        
    def test_each_product_has_review(self):
        url = reverse('catalogue:detail', kwargs=self.kwargs)
        response = self.client.get(url)
        self.assertEquals(200, response.status_code)
    
    def test_user_can_add_product_review(self):
        kwargs = {
                'product_slug': self.product.slug,
                'product_pk': str(self.product.id)}
        url = reverse('catalogue:reviews-add', kwargs=kwargs)
        self.client.login(username='testuser', password='secret')
        response = self.client.get(url)
        self.assertEquals(200, response.status_code)
        # check necessary review fields for logged in user
        self.assertContains(response, 'title')
        self.assertContains(response, 'score')
        # check additional fields for anonymous user
        self.client.login(username=None)
        response = self.client.get(url)
        self.assertContains(response, 'name')
        self.assertContains(response, 'email')




from django.contrib import admin

from oscar.core.loading import import_module
models = import_module('shipping.models', ['OrderAndItemLevelChargeMethod'])

class MethodAdmin(admin.ModelAdmin):
    exclude = ('code',)
    list_display = ('name', 'description', 'price_per_order', 'price_per_item', 'free_shipping_threshold')

admin.site.register(models.OrderAndItemLevelChargeMethod, MethodAdmin)



from django.contrib import admin

from oscar.apps.catalogue.reviews.models import ProductReview, Vote


class ProductReviewAdmin(admin.ModelAdmin):
    list_display = ('product', 'title', 'score', 'status', 'total_votes', 'delta_votes', 'date_created')
    readonly_fields = ('total_votes', 'delta_votes')


class VoteAdmin(admin.ModelAdmin):
    list_display = ('review', 'user', 'delta', 'date_created')

admin.site.register(ProductReview, ProductReviewAdmin)
admin.site.register(Vote, VoteAdmin)



from django.conf.urls.defaults import patterns, url
from django.contrib.admin.views.decorators import staff_member_required
from haystack.query import SearchQuerySet

from oscar.core.application import Application
from oscar.apps.reports.views import DashboardView


class ReportsApplication(Application):
    name = 'reports'
    
    dashboard_view = DashboardView

    def get_urls(self):
        urlpatterns = patterns('',
            url(r'^$', staff_member_required(self.dashboard_view.as_view()), name='dashboard'),
        )
        return urlpatterns

application = ReportsApplication()


from oscar.apps.analytics.abstract_models import (AbstractProductRecord, AbstractUserRecord,
                                                  AbstractUserProductView, AbstractUserSearch)


class ProductRecord(AbstractProductRecord):
    pass


class UserRecord(AbstractUserRecord):
    pass


class UserProductView(AbstractUserProductView):
    pass


class UserSearch(AbstractUserSearch):
    pass


# Import receiver functions
from oscar.apps.analytics.receivers import *


from decimal import Decimal as D
import datetime

from django.utils import unittest

from oscar.test.helpers import create_product
from oscar.apps.partner.abstract_models import partner_wrappers


class DummyWrapper(object):
    
    def availability(self, stockrecord):
        return 'Dummy response'
    
    def dispatch_date(self, stockrecord):
        return "Another dummy response"


class StockRecordTests(unittest.TestCase):

    def setUp(self):
        self.product = create_product(price=D('10.00'), num_in_stock=10)
   
    def test_get_price_incl_tax_defaults_to_no_tax(self):
        self.assertEquals(D('10.00'), self.product.stockrecord.price_incl_tax)
        
    def test_get_price_excl_tax_returns_correct_value(self):
        self.assertEquals(D('10.00'), self.product.stockrecord.price_excl_tax)
        
    def test_net_stock_level_with_no_allocation(self):
        self.assertEquals(10, self.product.stockrecord.net_stock_level)
        
    def test_net_stock_level_with_allocation(self):
        self.product.stockrecord.allocate(5)
        self.assertEquals(10-5, self.product.stockrecord.net_stock_level)    
        

class DefaultWrapperTests(unittest.TestCase):
    
    def test_default_wrapper_for_in_stock(self):
        product = create_product(price=D('10.00'), partner="Acme", num_in_stock=10)  
        self.assertEquals("In stock (10 available)", product.stockrecord.availability)
        
    def test_default_wrapper_for_out_of_stock(self):
        product = create_product(price=D('10.00'), partner="Acme", num_in_stock=0)  
        self.assertEquals("Out of stock", product.stockrecord.availability)
        
    def test_dispatch_date_for_in_stock(self):
        product = create_product(price=D('10.00'), partner="Acme", num_in_stock=1)  
        tomorrow = datetime.date.today() + datetime.timedelta(days=1)    
        self.assertEquals(tomorrow, product.stockrecord.dispatch_date)
    
    def test_dispatch_date_for_out_of_stock(self):
        product = create_product(price=D('10.00'), partner="Acme", num_in_stock=0)  
        date = datetime.date.today() + datetime.timedelta(days=7)
        self.assertEquals(date, product.stockrecord.dispatch_date)
    
    
class CustomWrapperTests(unittest.TestCase):    

    def setUp(self):
        partner_wrappers['Acme'] = DummyWrapper()
        
    def tearDown(self):
        del partner_wrappers['Acme']

    def test_wrapper_availability_gets_called(self):
        product = create_product(price=D('10.00'), partner="Acme", num_in_stock=10)  
        self.assertEquals(u"Dummy response", unicode(product.stockrecord.availability))
        
    def test_wrapper_dispatch_date_gets_called(self):
        product = create_product(price=D('10.00'), partner="Acme", num_in_stock=10)  
        self.assertEquals("Another dummy response", product.stockrecord.dispatch_date)
               



from oscar.apps.catalogue.abstract_models import AbstractItem

from django.db import models

class Item(AbstractItem):
    batteries_needed = models.BooleanField(default=False)
    batteries_included = models.BooleanField(default=False)

# We have to import any oscar implementations after we define our own, otherwise Django picks up the wrong one
from oscar.apps.catalogue.models import ItemClass, AttributeType, AttributeValueOption, ProductAttributeValue, Option


"""
Special settings file for use when testing.  This specifies a SQLite 
database to use when running tests.

Just make sure you run the tests and specify this file:
> ./manage.py test -settings=test_settings
"""
import sys
sys.path.append("../..")

from settings import *

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3', 
        'NAME': ':memory:', 
        'USER': '',                      # Not used with sqlite3.
        'PASSWORD': '',                  # Not used with sqlite3.
        'HOST': '',                      # Set to empty string for localhost. Not used with sqlite3.
        'PORT': '',                      # Set to empty string for default. Not used with sqlite3.
    }
}



from oscar.apps.basket.abstract_models import (AbstractBasket, AbstractLine, AbstractLineAttribute,
                                               OPEN, MERGED, SAVED, SUBMITTED)


class InvalidBasketLineError(Exception):
    pass


class Basket(AbstractBasket):
    pass

    
class Line(AbstractLine):
    pass


class LineAttribute(AbstractLineAttribute):
    pass


from django.test import TestCase, Client
from django.core.urlresolvers import reverse

from django.core import management
from haystack import backend


class SuggestViewTest(TestCase):
    fixtures = ['sample-products']

    def setUp(self):
        #clear out existing index without prompting user and ensure that
        #fixtures are indexed
        self.client = Client()
        sb = backend.SearchBackend()
        sb.clear()
        management.call_command('update_index', verbosity=0) #silenced

    def test_term_in_fixtures_found(self):
        url = reverse('oscar-search-suggest')
        response = self.client.get(url, {'query_term': 'Pint'})
        self.assertEquals(200, response.status_code)
        self.assertTrue('Pint' in response.content) #ensuring we actually find a result in the response

class MultiFacetedSearchViewTest(TestCase):
    fixtures = ['sample-products']

    def setUp(self):
        #clear out existing index without prompting user and ensure that
        #fixtures are indexed
        self.client = Client()
        sb = backend.SearchBackend()
        sb.clear()
        management.call_command('update_index', verbosity=0) #silenced

    def test_with_query(self):
        url = reverse('oscar-search')
        response = self.client.get(url, {'q': 'Pint'})
        self.assertEquals(200, response.status_code)
        self.assertTrue('value="Pint"' in response.content) #ensuring query field is set
    
        



import datetime

from django.db import models


class ActiveOfferManager(models.Manager):
    u"""
    For searching/creating ACTIVE offers only.
    """
    
    def get_query_set(self):
        today = datetime.date.today()
        return super(ActiveOfferManager, self).get_query_set().filter(start_date__lte=today, end_date__gt=today)




from oscar.core.application import Application
from django.conf.urls.defaults import patterns, url, include
from oscar.apps.catalogue.reviews.views import ProductReviewDetail, CreateProductReview, CreateProductReviewComplete, ProductReviewList 

class ProductReviewsApplication(Application):
    name = None
    detail_view = ProductReviewDetail
    create_view = CreateProductReview
    create_complete_view = CreateProductReviewComplete
    list_view = ProductReviewList

    def get_urls(self):
        urlpatterns = patterns('',
            url(r'^(?P<pk>\d+)/$', self.detail_view.as_view(), name='reviews-detail'),
            url(r'^add/$', self.create_view.as_view(), name='reviews-add'),
            url(r'^add/complete/$', self.create_complete_view.as_view(), name='reviews-add-complete'),            
            url(r'^$', self.list_view.as_view(), name='reviews-list'),
        )
        return urlpatterns

application = ProductReviewsApplication()


from django import forms
from django.db.models import get_model

class SignedInUserProductReviewForm(forms.ModelForm):
    class Meta:
        model = get_model('reviews', 'productreview')
        fields = ('title', 'score', 'body')
        
        
class AnonymousUserProductReviewForm(forms.ModelForm):
    name = forms.CharField(required=True)
    email = forms.EmailField(required=True)
    
    class Meta:
        model = get_model('reviews', 'productreview')
        fields = ('title', 'score', 'body', 'name', 'email', 'homepage')


class VoteForm(forms.ModelForm):
    def clean(self):
        user = self.instance.user
        review = self.instance.review  
        if review.user == user:
            raise forms.ValidationError("You cannot vote on your own reviews!")
        return self.cleaned_data
    
    class Meta:
        model = get_model('reviews', 'vote')
        exclude = ('review', 'user',)


from django.contrib import admin

from oscar.core.loading import import_module
models = import_module('payment.models', ['Source', 'Transaction', 'SourceType'])

class SourceAdmin(admin.ModelAdmin):
    list_display = ('order', 'source_type', 'amount_allocated', 'amount_debited', 'balance', 'reference')


admin.site.register(models.Source, SourceAdmin)
admin.site.register(models.SourceType)
admin.site.register(models.Transaction)



import os
from decimal import Decimal as D
from django.test import TestCase
import logging

from oscar.apps.partner.utils import CatalogueImporter
from oscar.apps.partner.exceptions import ImportError
from oscar.apps.catalogue.models import ProductClass, Product
from oscar.apps.partner.models import Partner, StockRecord
from oscar.test.helpers import create_product

TEST_BOOKS_CSV = os.path.join(os.path.dirname(__file__), '../fixtures/books-small.csv')
TEST_BOOKS_SEMICOLON_CSV = os.path.join(os.path.dirname(__file__), '../fixtures/books-small-semicolon.csv')

class NullHandler(logging.Handler):
    def emit(self, record):
        pass

logger = logging.getLogger("Null")
logger.addHandler(NullHandler())


class CommandEdgeCasesTest(TestCase):

    def setUp(self):
        self.importer = CatalogueImporter(logger)

    def test_sending_no_file_argument_raises_exception(self):
        self.importer.afile = None
        with self.assertRaises(ImportError):
            self.importer.handle()

    def test_sending_directory_as_file_raises_exception(self):
        self.importer.afile = "/tmp"
        with self.assertRaises(ImportError):
            self.importer.handle()

    def test_importing_nonexistant_file_raises_exception(self):
        self.importer.afile = "/tmp/catalogue-import.zgvsfsdfsd"
        with self.assertRaises(ImportError):
            self.importer.handle()


class ImportSmokeTest(TestCase):
    
    # First row is:
    # "9780115531446","Prepare for Your Practical Driving Test",NULL,"Book","Gardners","9780115531446","10.32","6"
    #
    # Second row is (has no stock data):
    # "9780955337819","Better Photography",NULL,"Book"
    
    
    def setUp(self):
        self.importer = CatalogueImporter(logger)
        self.importer.handle(TEST_BOOKS_CSV)
        self.item = Product.objects.get(upc='9780115531446')
        
    def test_all_rows_are_imported(self):
        self.assertEquals(10, Product.objects.all().count())
        
    def test_class_is_created(self):
        try:
            ProductClass.objects.get(name="Book")
        except Product.DoesNotExist:
            self.fail()  
            
    def test_only_one_class_is_created(self):
        self.assertEquals(1, ProductClass.objects.all().count())
    
    def test_item_is_created(self):
        try:
            Product.objects.get(upc="9780115531446")
        except Product.DoesNotExist:
            self.fail()         
    
    def test_title_is_imported(self):
        self.assertEquals("Prepare for Your Practical Driving Test", self.item.title)
        
    def test_partner_is_created(self):
        try:
            Partner.objects.get(name="Gardners")
        except Product.DoesNotExist:
            self.fail() 
    
    def test_stockrecord_is_created(self):
        try:
            StockRecord.objects.get(partner_sku="9780115531446")
        except Product.DoesNotExist:
            self.fail()      
        
    def test_null_fields_are_skipped(self):
        self.assertEquals("", self.item.description)
        
    def test_price_is_imported(self):
        self.assertEquals(D('10.32'), self.item.stockrecord.price_excl_tax)
        
    def test_num_in_stock_is_imported(self):
        self.assertEquals(6, self.item.stockrecord.num_in_stock)
        
        
class ImportSemicolonDelimitedFileTest(TestCase):
    
    def setUp(self):
        self.importer = CatalogueImporter(logger, delimiter=";")
        
    def test_import(self):
        self.importer.handle(TEST_BOOKS_SEMICOLON_CSV)
    
    
class ImportWithFlushTest(TestCase):
    
    def setUp(self):
        self.importer = CatalogueImporter(logger, flush=True)

    def test_items_are_flushed_by_importer(self):
        upc = "0000000000000"
        create_product(price=D('10.00'), upc=upc)
        
        self.importer.handle(TEST_BOOKS_CSV)
        
        with self.assertRaises(Product.DoesNotExist):
            Product.objects.get(upc=upc)
   


from decimal import Decimal

class ShippingMethod(object):
    u"""
    Superclass for all shipping method objects
    """
    code = '__default__'
    name = 'Default shipping'
    description = ''
    
    def __init__(self):
        self.exempt_from_tax = False
    
    def set_basket(self, basket):
        self.basket = basket
    
    def basket_charge_incl_tax(self):
        pass
    
    def basket_charge_excl_tax(self):
        pass
    
    
class FreeShipping(ShippingMethod):
    u"""
    Simple method for free shipping
    """
    code = 'free-shipping'
    name = 'Free shipping'
    
    def basket_charge_incl_tax(self):
        return Decimal('0.00')
    
    def basket_charge_excl_tax(self):
        return Decimal('0.00')
    

class FixedPriceShipping(ShippingMethod):
    code = 'fixed-price-shipping'
    name = 'Fixed price shipping'
    
    def __init__(self, charge_incl_tax, charge_excl_tax=None):
        self.charge_incl_tax = charge_incl_tax
        if not charge_excl_tax:
            charge_excl_tax = charge_incl_tax
        self.charge_excl_tax = charge_excl_tax
    
    def basket_charge_incl_tax(self):
        return self.charge_incl_tax
    
    def basket_charge_excl_tax(self):
        return self.charge_excl_tax



from oscar.apps.partner.tests.model_tests import *
from oscar.apps.partner.tests.import_catalogue import *

__test__ = {
    'model_tests': [model_tests],
    'import_catalogue': [import_catalogue],
}


from django.forms import ModelForm
from django.db.models import get_model


class ShippingAddressForm(ModelForm):
    
    def __init__(self, *args, **kwargs):
        super(ShippingAddressForm,self ).__init__(*args, **kwargs)
        self.set_country_queryset() 
        
    def set_country_queryset(self):    
        self.fields['country'].queryset = get_model('address', 'country')._default_manager.filter(is_shipping_country=True)
    
    class Meta:
        model = get_model('order', 'shippingaddress')
        exclude = ('user', 'search_text')




import csv
from oscar.apps.reports.reports import ReportGenerator

from oscar.core.loading import import_module
import_module('reports.reports', ['ReportGenerator'], locals())
import_module('basket.models', ['Basket', 'OPEN', 'SUBMITTED'], locals())


class OpenBasketReportGenerator(ReportGenerator):
    
    filename_template = 'open-baskets-%s-%s.csv'
    code = 'open_baskets'
    description = 'Open baskets'
    
    def generate(self, response):
        writer = csv.writer(response)
        header_row = ['User ID',
                      'Username',
                      'Name',
                      'Email',
                      'Basket status',
                      'Num lines',
                      'Num items',
                      'Value',
                      'Date of creation',
                      'Time since creation',
                     ]
        writer.writerow(header_row)
        
        baskets = Basket._default_manager.filter(status=Basket.OPEN)
        for basket in baskets:
            if basket.owner:
                row = [basket.owner_id, basket.owner.username, basket.owner.get_full_name(). basket.owner.email,
                       basket.status, basket.num_lines,
                       basket.num_items, basket.total_incl_tax, 
                       basket.date_created, basket.time_since_creation]
            else:
                row = [basket.owner_id, None, None, None, basket.status, basket.num_lines,
                       basket.num_items, basket.total_incl_tax, 
                       basket.date_created, basket.time_since_creation]
            writer.writerow(row)


class SubmittedBasketReportGenerator(ReportGenerator):
    
    filename_template = 'submitted_baskets-%s-%s.csv'
    code = 'submitted_baskets'
    description = 'Submitted baskets'
    
    def generate(self, response):
        writer = csv.writer(response)
        header_row = ['User ID',
                      'User',
                      'Basket status',
                      'Num lines',
                      'Num items',
                      'Value',
                      'Time between creation and submission',
                     ]
        writer.writerow(header_row)
        
        baskets = Basket._default_manager.filter(status=Basket.SUBMITTED)
        for basket in baskets:
            row = [basket.owner_id, basket.owner, basket.status, basket.num_lines,
                   basket.num_items, basket.total_incl_tax, 
                   basket.date_created, basket.time_before_submit]
            writer.writerow(row)



from oscar.apps.voucher.abstract_models import AbstractVoucher, AbstractVoucherApplication


class Voucher(AbstractVoucher):
    pass


class VoucherApplication(AbstractVoucherApplication):
    pass





from decimal import Decimal as D
import httplib

from django.test import TestCase
from django.test.client import Client
from django.core.urlresolvers import reverse

from oscar.test.helpers import create_product

        
class CheckoutViewsTest(TestCase):
    
    fixtures = ['example-shipping-charges.json']
    
    def setUp(self):
        self.client = Client()
        super(CheckoutViewsTest, self).setUp()
    
    def test_user_address_views_require_a_login(self):
        urls = [reverse('checkout:user-address-create'),
                reverse('checkout:user-address-update', kwargs={'pk': 1}),
                reverse('checkout:user-address-delete', kwargs={'pk': 1}),]
        for url in urls:
            response = self.client.get(url)
            self.assertEqual(httplib.FOUND, response.status_code)
    
    def _test_anonymous_checkout(self):
        
        # Add a product to the basket
        p = create_product(price=D('10.00'))
        response = self.client.post(reverse('basket:add'), {'action': 'add', 
                                                              'product_id': str(p.id),
                                                              'quantity': 1})
        self.assertEqual(302, response.status_code)
        
        # Submit shipping address
        response = self.client.post(reverse('oscar-checkout-shipping-address'), 
                                    {'last_name': 'Smith',
                                     'line1': '1 Portland Street',
                                     'postcode': 'N12 9ET',
                                     'country': 'GB'})
        self.assertEqual(302, response.status_code)
        
        # Choose shipping method
        response = self.client.post(reverse('oscar-checkout-shipping-method'),
                                    {'method_code': 'royal-mail-first-class'})
        self.assertEqual(302, response.status_code)
        
        # Shipping method
        response = self.client.get(reverse('oscar-checkout-payment-method'))
        self.assertEqual(302, response.status_code)
        
        # View preview
        response = self.client.get(reverse('oscar-checkout-preview'))
        self.assertEqual(302, response.status_code)
        
        # Submit
        response = self.client.post(reverse('oscar-checkout-payment-details'), {})
        self.assertEqual(302, response.status_code)
        
        
        
        
        
        



import twill
from StringIO import StringIO

from django.test import TestCase
from django.core.servers.basehttp import AdminMediaHandler
from django.core.handlers.wsgi import WSGIHandler
from django.core.urlresolvers import reverse

from oscar.apps.catalogue.models import ProductClass, Product
from oscar.apps.partner.models import Partner, StockRecord

def create_product(price=None, title="Dummy title", product_class="Dummy item class", 
                   partner="Dummy partner", upc="dummy_101", num_in_stock=10):
    """
    Helper method for creating products that are used in tests.
    """
    ic,_ = ProductClass._default_manager.get_or_create(name=product_class)
    item = Product._default_manager.create(title=title, product_class=ic, upc=upc)
    if price:
        partner,_ = Partner._default_manager.get_or_create(name=partner)
        sr = StockRecord._default_manager.create(product=item, partner=partner, 
                                                 price_excl_tax=price, num_in_stock=num_in_stock)
    return item


class TwillTestCase(TestCase):
    """
    Simple wrapper around Twill to make writing TestCases easier.

    Commands availabel through self.command are:
    - go        -> visit a URL
    - back      -> back to previous URL
    - reload    -> reload URL
    - follow    -> follow a given link
    - code      -> assert the HTTP response code
    - find      -> assert page contains some string
    - notfind   -> assert page does not contain
    - title     -> assert page title
    """

    HOST = '127.0.0.1'
    PORT = 8080

    def setUp(self):
        app = AdminMediaHandler(WSGIHandler())
        twill.add_wsgi_intercept(self.HOST, self.PORT, lambda: app)
        twill.set_output(StringIO())
        self.command = twill.commands

    def tearDown(self):
        twill.remove_wsgi_intercept(self.HOST, self.PORT)

    def reverse(self, url_name, *args, **kwargs):
        """
        Custom 'reverse' function that includes the protocol and host
        """
        return 'http://%s:%d%s' % (self.HOST, self.PORT, reverse(url_name, *args, **kwargs))

    def visit(self, url_name, *args,**kwargs):
        self.command.go(self.reverse(url_name, *args, **kwargs))

    def assertResponseCodeIs(self, code):
        self.command.code(code)

    def assertPageContains(self, regexp):
        self.command.find(regexp)

    def assertPageDoesNotContain(self, regexp):
        self.command.notfind(regexp)

    def assertPageTitleMatches(self, regexp):
        self.command.title(regexp)




from django.contrib import admin
from django.db.models import get_model


class BasketAdmin(admin.ModelAdmin):
    read_only_fields = ('date_merged', 'date_submitted')


admin.site.register(get_model('basket', 'basket'), BasketAdmin)
admin.site.register(get_model('basket', 'line'))
admin.site.register(get_model('basket', 'LineAttribute'))



import logging
import sys
import os
from optparse import make_option

from django.core.management.base import BaseCommand, CommandError

from oscar.core.loading import import_module
import_module('catalogue.utils', ['Importer'], locals())


class Command(BaseCommand):
    
    args = '/path/to/folder'
    help = 'For importing product images from a folder'
    
    option_list = BaseCommand.option_list + (
        make_option('--filename',
            dest='filename',
            default='upc',
            help='Product field to lookup from image filename'),
        )

    def handle(self, *args, **options):
        if len(args) != 1:
            raise CommandError('Command requires a path to a single folder') 
        
        logger = self._get_logger()
        logger.info("Starting image import...")       
        
        dirname = args[0]
        importer = Importer(logger, field=options.get('filename'))
        importer.handle(dirname)
            
    def _get_logger(self):
        logger = logging.getLogger('oscar.apps.image')
        stream = logging.StreamHandler(self.stdout)
        logger.addHandler(stream)
        logger.setLevel(logging.DEBUG)
        return logger
            


from datetime import date, datetime

from django import forms

from oscar.core.loading import import_module
report_utils = import_module('reports.utils', ['GeneratorRepository'])

class ReportForm(forms.Form):
    
    generators = report_utils.GeneratorRepository().get_report_generators()
    
    type_choices = []
    for generator in generators:
        type_choices.append((generator.code, generator.description))
    report_type = forms.ChoiceField(widget=forms.Select(), choices=type_choices)
    start_date = forms.DateField(widget=forms.widgets.DateInput(format="%d/%m/%Y"), 
                                 help_text='Format dd/mm/YYYY',
                                 input_formats=['%d/%m/%y', '%d/%m/%Y'])
    end_date = forms.DateField(widget=forms.widgets.DateInput(format="%d/%m/%Y"), 
                               help_text='Format dd/mm/YYYY',
                               input_formats=['%d/%m/%y', '%d/%m/%Y'])
    
    def clean(self):
        if 'start_date' in self.cleaned_data and 'end_date' in self.cleaned_data and self.cleaned_data['start_date'] > self.cleaned_data['end_date']:
            raise forms.ValidationError("Your start date must be before your end date")
        return self.cleaned_data
   


import datetime

from django.test import TestCase

from oscar.apps.voucher.models import Voucher


class VoucherTest(TestCase):
    
    def test_is_active(self):
        start = datetime.date(2011, 01, 01)
        test = datetime.date(2011, 01, 10)
        end = datetime.date(2011, 02, 01)
        voucher = Voucher(start_date=start, end_date=end)
        self.assertTrue(voucher.is_active(test))

    def test_is_inactive(self):
        start = datetime.date(2011, 01, 01)
        test = datetime.date(2011, 03, 10)
        end = datetime.date(2011, 02, 01)
        voucher = Voucher(start_date=start, end_date=end)
        self.assertFalse(voucher.is_active(test))
        
    def test_codes_are_saved_as_uppercase(self):
        start = datetime.date(2011, 01, 01)
        end = datetime.date(2011, 02, 01)
        voucher = Voucher(name="Dummy voucher", code="lowercase", start_date=start, end_date=end)
        voucher.save()
        self.assertEquals("LOWERCASE", voucher.code)
        

    
   



from django.contrib.sites.models import Site

from oscar.core.loading import import_module
import_module('order.models', ['ShippingAddress', 'Order', 'Line', 
                               'LinePrice', 'LineAttribute', 'OrderDiscount'], locals())
import_module('order.signals', ['order_placed'], locals())


class OrderNumberGenerator(object):
    """
    Simple object for generating order numbers.

    We need this as the order number is often required for payment
    which takes place before the order model has been created.
    """

    def order_number(self, basket):
        """
        Return an order number for a given basket
        """
        return 100000 + basket.id


class OrderCreator(object):
    """
    Places the order by writing out the various models
    """
    
    def place_order(self, user, basket, shipping_address, shipping_method, 
                    billing_address, total_incl_tax, total_excl_tax, 
                    order_number=None, status=None, **kwargs):
        """
        Placing an order involves creating all the relevant models based on the
        basket and session data.
        """
        if not order_number:
            generator = OrderNumberGenerator()
            order_number = generator.order_number(basket)
        order = self.create_order_model(user, basket, shipping_address, 
                                        shipping_method, billing_address, total_incl_tax, 
                                        total_excl_tax, order_number, status, **kwargs)
        for line in basket.all_lines():
            self.create_line_models(order, line)
            self.update_stock_records(line)
        for discount in basket.get_discounts():
            self.create_discount_model(order, discount)
        for voucher in basket.vouchers.all():
            self.record_voucher_usage(order, voucher, user)
        
        # Send signal for analytics to pick up
        order_placed.send(sender=self, order=order, user=user)
        
        return order
        
    def create_order_model(self, user, basket, shipping_address, shipping_method, 
                           billing_address, total_incl_tax, total_excl_tax, 
                           order_number, status, **extra_order_fields):
        """
        Creates an order model.
        """
        order_data = {'basket': basket,
                      'number': order_number,
                      'site': Site._default_manager.get_current(),
                      'total_incl_tax': total_incl_tax,
                      'total_excl_tax': total_excl_tax,
                      'shipping_address': shipping_address,
                      'shipping_incl_tax': shipping_method.basket_charge_incl_tax(),
                      'shipping_excl_tax': shipping_method.basket_charge_excl_tax(),
                      'shipping_method': shipping_method.name}
        if billing_address:
            order_data['billing_address'] = billing_address
        if user.is_authenticated():
            order_data['user_id'] = user.id
        if status:
            order_data['status'] = status
        if extra_order_fields:
            order_data.update(extra_order_fields)
        order = Order(**order_data)
        order.save()
        return order
    
    def get_partner_for_product(self, product):
        """
        Return the partner for a product
        """
        if product.has_stockrecord:
            return product.stockrecord.partner
        raise AttributeError("No partner found for product '%s'" % product)
    
    def create_line_models(self, order, basket_line, extra_line_fields=None):
        """
        Create the batch line model.
        
        You can set extra fields by passing a dictionary as the extra_line_fields value
        """
        partner = self.get_partner_for_product(basket_line.product)
        stockrecord = basket_line.product.stockrecord
        line_data = {'order': order,
                      # Partner details
                      'partner': partner,
                      'partner_name': partner.name,
                      'partner_sku': stockrecord.partner_sku,
                      # Product details
                      'product': basket_line.product, 
                      'title': basket_line.product.get_title(),
                      'quantity': basket_line.quantity,
                      # Price details 
                      'line_price_excl_tax': basket_line.line_price_excl_tax_and_discounts, 
                      'line_price_incl_tax': basket_line.line_price_incl_tax_and_discounts,
                      'line_price_before_discounts_excl_tax': basket_line.line_price_excl_tax,
                      'line_price_before_discounts_incl_tax': basket_line.line_price_incl_tax,
                      # Reporting details
                      'unit_cost_price': stockrecord.cost_price,
                      'unit_price_incl_tax': basket_line.unit_price_incl_tax,
                      'unit_price_excl_tax': basket_line.unit_price_excl_tax,
                      'unit_retail_price': stockrecord.price_retail,
                      # Shipping details
                      'est_dispatch_date':  basket_line.product.stockrecord.dispatch_date
                     }
        if extra_line_fields:
            line_data.update(extra_line_fields)
        
        order_line = Line._default_manager.create(**line_data)
        self.create_line_price_models(order, order_line, basket_line)
        self.create_line_attributes(order, order_line, basket_line)
        
    def update_stock_records(self, line):
        line.product.stockrecord.allocate(line.quantity)    
        
    def create_line_price_models(self, order, order_line, basket_line):
        """
        Creates the batch line price models
        """
        breakdown = basket_line.get_price_breakdown()
        for price_incl_tax, price_excl_tax, quantity in breakdown:
            LinePrice._default_manager.create(order=order,
                                              line=order_line, 
                                              quantity=quantity, 
                                              price_incl_tax=price_incl_tax,
                                              price_excl_tax=price_excl_tax)
    
    def create_line_attributes(self, order, order_line, basket_line):
        """
        Creates the batch line attributes.
        """
        for attr in basket_line.attributes.all():
            LineAttribute._default_manager.create(line=order_line,
                                                   option=attr.option, 
                                                   type=attr.option.code,
                                                   value=attr.value)
            
    def create_discount_model(self, order, discount):
        """
        Creates an order discount model for each discount attached to the basket.
        """
        order_discount = OrderDiscount(order=order, offer=discount['offer'], amount=discount['discount'])
        if discount['voucher']:
            order_discount.voucher = discount['voucher']
            order_discount.voucher_code = discount['voucher'].code
        order_discount.save()
        
    def record_voucher_usage(self, order, voucher, user):
        """
        Updates the models that care about this voucher.
        """
        voucher.record_usage(order, user)
        voucher.num_orders += 1
        voucher.save()



import Image
import ImageChops
import math


def hex_to_color(hex):
    r = int(hex[0:2], 16)
    g = int(hex[2:4], 16)
    b = int(hex[4:6], 16)

    if len(hex) == 8:
        a = int(hex[6:8], 16)
    else:
        a = 255

    return (r, g, b, a)


class BaseModification(object):
    def __init__(self, image, params):
        self._params = params
        self._image = image

    def apply(self):
        pass


class AutotrimMod(BaseModification):
    def apply(self):
        keys = self._params.keys()
        if ('width' in keys or 'height' in keys) and not 'crop' in keys:
            if 'bgcolor' in keys:
                bgcolor = hex_to_color(self._params['bgcolor'])
            else:
                bgcolor = (255, 255, 255, 255)

            bg = Image.new(self._image.mode, self._image.size, bgcolor)
            diff = ImageChops.difference(self._image, bg)
            bbox = diff.getbbox()

            return self._image.crop(bbox)
        return self._image


class CropMod(BaseModification):
    def apply(self):
        if 'crop' in self._params.keys():
            crop = self._params['crop']
            bounds = tuple(int(x) for x in crop.split(','))
            return self._image.crop(bounds)
        return self._image


class ResizeMod(BaseModification):
    def apply(self):
        keys = self._params.keys()
        if 'width' in keys or 'height' in keys:
            w = self._params.get('width', None)
            h = self._params.get('height', None)

            target_w = float(w) if w else None
            target_h = float(h) if h else None

            source_w, source_h = [float(v) for v in self._image.size]

            scale = 1.0

            if target_w:
                temp_scale = target_w / source_w
                if temp_scale < scale:
                    scale = temp_scale
            if target_h:
                temp_scale = target_h / source_h
                if temp_scale < scale:
                    scale = temp_scale

            if scale < 1.0:
                new_size = (int(round(source_w * scale)),
                            int(round(source_h * scale)))
                self._image = self._image.resize(new_size, Image.ANTIALIAS)

            new_w = int(target_w if target_w else self._image.size[0])
            new_h = int(target_h if target_h else self._image.size[1])

            if 'bgcolor' in keys:
                bgcolor = hex_to_color(self._params['bgcolor'])
            else:
                bgcolor = (255, 255, 255, 255)

            bg = Image.new(self._image.mode, (new_w, new_h), bgcolor)

            left = int(math.floor(float(new_w - self._image.size[0]) / 2))
            top = int(math.floor(float(new_h - self._image.size[1]) / 2))

            bg.paste(self._image, (left, top))

            return bg
        return self._image



def dataProvider(fn_data_provider):
    """
    Data provider decorator, allows another callable to provide the data for the test.  
    This is a nice feature from PHPUnit which is very useful.  Am sticking with the JUnit style
    naming as unittest does this already.
    
    Implementation based on http://melp.nl/2011/02/phpunit-style-dataprovider-in-python-unit-test/#more-525
    """
    def test_decorator(test_method):
        def execute_test_method_with_each_data_set(self):
            for data in fn_data_provider():
                if len(data) == 2 and isinstance(data[0], tuple) and isinstance(data[1], dict):
                    # Both args and kwargs being provided
                    args, kwargs = data[:]
                else:
                    args, kwargs = data, {}
                try:
                    test_method(self, *args, **kwargs)
                except AssertionError, e:
                    self.fail("%s (Provided data: %s, %s)" % (e, args, kwargs))
        return execute_test_method_with_each_data_set
    return test_decorator


from itertools import chain
from decimal import Decimal as D

from django.db import models
from django.contrib.auth.models import User
from django.template.defaultfilters import slugify
from django.utils.translation import ugettext_lazy as _
from django.db.models import Sum
from django.core.exceptions import ObjectDoesNotExist


class AbstractOrder(models.Model):
    """
    The main order model
    """
    
    number = models.CharField(_("Order number"), max_length=128, db_index=True)
    # We track the site that each order is placed within
    site = models.ForeignKey('sites.Site')
    basket = models.ForeignKey('basket.Basket', null=True, blank=True)
    # Orders can be anonymous so we don't always have a customer ID
    user = models.ForeignKey(User, related_name='orders', null=True, blank=True)
    # Billing address is not always required (eg paying by gift card)
    billing_address = models.ForeignKey('order.BillingAddress', null=True, blank=True)
    
    # Total price looks like it could be calculated by adding up the
    # prices of the associated lines, but in some circumstances extra
    # order-level charges are added and so we need to store it separately
    total_incl_tax = models.DecimalField(_("Order total (inc. tax)"), decimal_places=2, max_digits=12)
    total_excl_tax = models.DecimalField(_("Order total (excl. tax)"), decimal_places=2, max_digits=12)
    
    # Shipping charges
    shipping_incl_tax = models.DecimalField(_("Shipping charge (inc. tax)"), decimal_places=2, max_digits=12, default=0)
    shipping_excl_tax = models.DecimalField(_("Shipping charge (excl. tax)"), decimal_places=2, max_digits=12, default=0)
    
    # Not all lines are actually shipped (such as downloads), hence shipping address
    # is not mandatory.
    shipping_address = models.ForeignKey('order.ShippingAddress', null=True, blank=True)
    shipping_method = models.CharField(_("Shipping method"), max_length=128, null=True, blank=True)
    
    # Use this field to indicate that an order is on hold / awaiting payment
    status = models.CharField(_("Status"), max_length=100, null=True, blank=True)
    
    # Index added to this field for reporting
    date_placed = models.DateTimeField(auto_now_add=True, db_index=True)
    
    @property
    def basket_total_incl_tax(self):
        """
        Return basket total including tax
        """
        return self.total_incl_tax - self.shipping_incl_tax
    
    @property
    def basket_total_excl_tax(self):
        """
        Return basket total excluding tax
        """
        return self.total_excl_tax - self.shipping_excl_tax
    
    @property
    def total_before_discounts_incl_tax(self):
        total = D('0.00')
        for line in self.lines.all():
            total += line.line_price_before_discounts_incl_tax
        total += self.shipping_incl_tax
        return total
    
    @property
    def total_before_discounts_excl_tax(self):
        total = D('0.00')
        for line in self.lines.all():
            total += line.line_price_before_discounts_excl_tax
        total += self.shipping_excl_tax
        return total
    
    @property
    def total_discount_incl_tax(self):
        """
        The amount of discount this order received
        """
        discount = D('0.00')
        for line in self.lines.all():
            discount += line.discount_incl_tax
        return discount
    
    @property
    def total_discount_excl_tax(self):
        discount = D('0.00')
        for line in self.lines.all():
            discount += line.discount_excl_tax
        return discount
    
    @property
    def total_tax(self):
        return self.total_incl_tax - self.total_excl_tax
    
    @property
    def num_lines(self):
        return self.lines.count()
    
    @property
    def num_items(self):
        u"""
        Returns the number of items in this order.
        """
        num_items = 0
        for line in self.lines.all():
            num_items += line.quantity
        return num_items
    
    @property
    def shipping_status(self):
        events = self.shipping_events.all()
        if not len(events):
            return ''
        
        # Collect all events by event-type
        map = {}
        for event in events:
            event_name = event.event_type.name
            if event_name not in map:
                map[event_name] = []
            map[event_name] = list(chain(map[event_name], event.line_quantities.all()))
        
        # Determine last complete event
        status = _("In progress")
        for event_name, event_line_quantities in map.items():
            if self._is_event_complete(event_line_quantities):
                status = event_name
        return status
    
    def _is_event_complete(self, event_quantites):
        # Form map of line to quantity
        map = {}
        for event_quantity in event_quantites:
            line_id = event_quantity.line_id
            map.setdefault(line_id, 0)
            map[line_id] += event_quantity.quantity
        
        for line in self.lines.all():
            if map[line.id] != line.quantity:
                return False
        return True
    
    class Meta:
        abstract = True
        ordering = ['-date_placed',]
        permissions = (
            ("can_view", "Can view orders (eg for reporting)"),
        )
    
    def __unicode__(self):
        return u"#%s" % (self.number,)


class AbstractOrderNote(models.Model):
    """
    A note against an order.
    
    This are often used for audit purposes too.  IE, whenever an admin
    makes a change to an order, we create a note to record what happened.
    """
    order = models.ForeignKey('order.Order', related_name="notes")
    
    # These are sometimes programatically generated so don't need a 
    # user everytime
    user = models.ForeignKey('auth.User', null=True)
    
    # We allow notes to be classified although this isn't always needed
    INFO, WARNING, ERROR = 'Info', 'Warning', 'Error'
    note_type = models.CharField(max_length=128, null=True)
    
    message = models.TextField()
    date = models.DateTimeField(auto_now_add=True)
    
    class Meta:
        abstract = True
        
    def __unicode__(self):
        return u"'%s' (%s)" % (self.message[0:50], self.user)


class AbstractCommunicationEvent(models.Model):
    """
    An order-level event involving a communication to the customer, such
    as an confirmation email being sent.
    """
    order = models.ForeignKey('order.Order', related_name="communication_events")
    type = models.ForeignKey('customer.CommunicationEventType')
    date = models.DateTimeField(auto_now_add=True)
    
    class Meta:
        abstract = True
        
    def __unicode__(self):
        return u"'%s' event for order #%s" % (self.type.name, self.order.number)
    
        
class AbstractLine(models.Model):
    """
    A order line (basically a product and a quantity)
    
    Not using a line model as it's difficult to capture and payment 
    information when it splits across a line.
    """
    order = models.ForeignKey('order.Order', related_name='lines')
    
    # We store the partner, their SKU and the title for cases where the product has been
    # deleted from the catalogue.  We also store the partner name in case the partner
    # gets deleted at a later date.
    partner = models.ForeignKey('partner.Partner', related_name='order_lines', blank=True, null=True, on_delete=models.SET_NULL)
    partner_name = models.CharField(_("Partner name"), max_length=128)
    partner_sku = models.CharField(_("Partner SKU"), max_length=128)
    title = models.CharField(_("Title"), max_length=255)
    
    # We don't want any hard links between orders and the products table so we allow
    # this link to be NULLable.
    product = models.ForeignKey('catalogue.Product', on_delete=models.SET_NULL, blank=True, null=True)
    quantity = models.PositiveIntegerField(default=1)
    
    # Price information (these fields are actually redundant as the information
    # can be calculated from the LinePrice models
    line_price_incl_tax = models.DecimalField(decimal_places=2, max_digits=12)
    line_price_excl_tax = models.DecimalField(decimal_places=2, max_digits=12)
    
    # Price information before discounts are applied
    line_price_before_discounts_incl_tax = models.DecimalField(decimal_places=2, max_digits=12)
    line_price_before_discounts_excl_tax = models.DecimalField(decimal_places=2, max_digits=12)

    # REPORTING FIELDS        
    # Cost price (the price charged by the fulfilment partner for this product).
    unit_cost_price = models.DecimalField(decimal_places=2, max_digits=12, blank=True, null=True)
    # Normal site price for item (without discounts)
    unit_price_incl_tax = models.DecimalField(decimal_places=2, max_digits=12, blank=True, null=True)
    unit_price_excl_tax = models.DecimalField(decimal_places=2, max_digits=12, blank=True, null=True)
    # Retail price at time of purchase
    unit_retail_price = models.DecimalField(decimal_places=2, max_digits=12, blank=True, null=True)
    
    # Partner information
    partner_line_reference = models.CharField(_("Partner reference"), max_length=128, blank=True, null=True,
        help_text=_("This is the item number that the partner uses within their system"))
    partner_line_notes = models.TextField(blank=True, null=True)
    
    # Partners often want to assign some status to each line to help with their own 
    # business processes.
    status = models.CharField(_("Status"), max_length=255, null=True, blank=True)
    
    # Estimated dispatch date - should be set at order time
    est_dispatch_date = models.DateField(blank=True, null=True)
    
    @property
    def description(self):
        """
        Returns a description of this line including details of any 
        line attributes.
        """
        desc = self.title
        ops = []
        for attribute in self.attributes.all():
            ops.append("%s = '%s'" % (attribute.type, attribute.value))
        if ops:
            desc = "%s (%s)" % (desc, ", ".join(ops))
        return desc
    
    @property
    def discount_incl_tax(self):
        return self.line_price_before_discounts_incl_tax - self.line_price_incl_tax
    
    @property
    def discount_excl_tax(self):
        return self.line_price_before_discounts_excl_tax - self.line_price_excl_tax
    
    @property
    def shipping_status(self):
        u"""Returns a string summary of the shipping status of this line"""
        status_map = self._shipping_event_history()
        if not status_map:
            return ''
        
        events = []    
        last_complete_event_name = None
        for event_dict in status_map:
            if event_dict['quantity'] == self.quantity:
                events.append(event_dict['name'])
                last_complete_event_name = event_dict['name']    
            else:
                events.append("%s (%d/%d items)" % (event_dict['name'], 
                                                    event_dict['quantity'], self.quantity))
        
        if last_complete_event_name == status_map[-1]['name']:
            return last_complete_event_name
            
        return ', '.join(events)
    
    def has_shipping_event_occurred(self, event_type):
        u"""Checks whether this line has passed a given shipping event"""
        for event_dict in self._shipping_event_history():
            if event_dict['name'] == event_type.name and event_dict['quantity'] == self.quantity:
                return True
        return False
    
    @property
    def is_product_deleted(self):
        return self.product == None
    
    def _shipping_event_history(self):
        u"""
        Returns a list of shipping events"""
        status_map = {}
        for event in self.shippingevent_set.all():
            event_name = event.event_type.name
            event_quantity = event.line_quantities.get(line=self).quantity
            if event_name in status_map:
                status_map[event_name]['quantity'] += event_quantity
            else:
                status_map[event_name] = {'name': event_name, 'quantity': event_quantity}
        return list(status_map.values())
    
    class Meta:
        abstract = True
        verbose_name_plural = _("Order lines")
        
    def __unicode__(self):
        try:
            title = self.product.title
        except ObjectDoesNotExist:
            title = '<missing product>'
        
        return u"Product '%s', quantity '%s'" % (title, self.quantity)
    
    
class AbstractLineAttribute(models.Model):
    u"""An attribute of a line."""
    line = models.ForeignKey('order.Line', related_name='attributes')
    option = models.ForeignKey('catalogue.Option', null=True, on_delete=models.SET_NULL, related_name="line_attributes")
    type = models.CharField(_("Type"), max_length=128)
    value = models.CharField(_("Value"), max_length=255)    
    
    class Meta:
        abstract = True
        
    def __unicode__(self):
        return "%s = %s" % (self.type, self.value)
    
    
class AbstractLinePrice(models.Model):
    u"""
    For tracking the prices paid for each unit within a line.
    
    This is necessary as offers can lead to units within a line 
    having different prices.  For example, one product may be sold at
    50% off as it's part of an offer while the remainder are full price.
    """
    order = models.ForeignKey('order.Order', related_name='line_prices')
    line = models.ForeignKey('order.Line', related_name='prices')
    quantity = models.PositiveIntegerField(default=1)
    price_incl_tax = models.DecimalField(decimal_places=2, max_digits=12)
    price_excl_tax = models.DecimalField(decimal_places=2, max_digits=12)
    shipping_incl_tax = models.DecimalField(decimal_places=2, max_digits=12, default=0)
    shipping_excl_tax = models.DecimalField(decimal_places=2, max_digits=12, default=0)
    
    class Meta:
        abstract = True
        
    def __unicode__(self):
        return u"Line '%s' (quantity %d) price %s" % (self.line, self.quantity, self.price_incl_tax)
   
   
# PAYMENT EVENTS   


class AbstractPaymentEventType(models.Model):
    """
    Payment events are things like 'Paid', 'Failed', 'Refunded'
    """
    name = models.CharField(max_length=128)
    code = models.SlugField(max_length=128)
    sequence_number = models.PositiveIntegerField(default=0)
    
    def save(self, *args, **kwargs):
        if not self.code:
            self.code = slugify(self.name)
        super(AbstractPaymentEventType, self).save(*args, **kwargs)
    
    class Meta:
        abstract = True
        verbose_name_plural = _("Payment event types")
        ordering = ('sequence_number',)
        
    def __unicode__(self):
        return self.name
   
   
class AbstractPaymentEvent(models.Model):    
    """
    An event is something which happens to a line such as
    payment being taken for 2 items, or 1 item being dispatched.
    """
    order = models.ForeignKey('order.Order', related_name='payment_events')
    lines = models.ManyToManyField('order.Line', through='PaymentEventQuantity')
    event_type = models.ForeignKey('order.PaymentEventType')
    date = models.DateTimeField(auto_now_add=True)
    
    class Meta:
        abstract = True
        verbose_name_plural = _("Payment events")
        
    def __unicode__(self):
        return u"Payment event for order #%d" % self.order


class PaymentEventQuantity(models.Model):
    """
    A "through" model linking lines to payment events
    """
    event = models.ForeignKey('order.PaymentEvent', related_name='line_quantities')
    line = models.ForeignKey('order.Line')
    quantity = models.PositiveIntegerField()


class AbstractShippingEvent(models.Model):    
    u"""
    An event is something which happens to a group of lines such as
    1 item being dispatched.
    """
    order = models.ForeignKey('order.Order', related_name='shipping_events')
    lines = models.ManyToManyField('order.Line', through='ShippingEventQuantity')
    event_type = models.ForeignKey('order.ShippingEventType')
    notes = models.TextField(_("Event notes"), blank=True, null=True,
        help_text="This could be the dispatch reference, or a tracking number")
    date = models.DateTimeField(auto_now_add=True)
    
    class Meta:
        abstract = True
        verbose_name_plural = _("Shipping events")
        ordering = ['-date']
        
    def __unicode__(self):
        return u"Order #%s, type %s" % (
            self.order.number, self.event_type)
        
    def num_affected_lines(self):
        return self.lines.count()


class ShippingEventQuantity(models.Model):
    u"""A "through" model linking lines to shipping events"""
    event = models.ForeignKey('order.ShippingEvent', related_name='line_quantities')
    line = models.ForeignKey('order.Line')
    quantity = models.PositiveIntegerField()

    def _check_previous_events_are_complete(self):
        u"""Checks whether previous shipping events have passed"""
        previous_events = ShippingEventQuantity._default_manager.filter(line=self.line, 
                                                                        event__event_type__sequence_number__lt=self.event.event_type.sequence_number)
        self.quantity = int(self.quantity)
        for event_quantities in previous_events:
            if event_quantities.quantity < self.quantity:
                raise ValueError("Invalid quantity (%d) for event type (a previous event has not been fully passed)" % self.quantity)

    def _check_new_quantity(self):
        quantity_row = ShippingEventQuantity._default_manager.filter(line=self.line, 
                                                                     event__event_type=self.event.event_type).aggregate(Sum('quantity'))
        previous_quantity = quantity_row['quantity__sum']
        if previous_quantity == None:
            previous_quantity = 0
        if previous_quantity + self.quantity > self.line.quantity:
            raise ValueError("Invalid quantity (%d) for event type (total exceeds line total)" % self.quantity)                                                        

    def save(self, *args, **kwargs):
        # Default quantity to full quantity of line
        if not self.quantity:
            self.quantity = self.line.quantity
        self._check_previous_events_are_complete()
        self._check_new_quantity()
        super(ShippingEventQuantity, self).save(*args, **kwargs)
        
    def __unicode__(self):
        return "%s - quantity %d" % (self.line.product, self.quantity)


class AbstractShippingEventType(models.Model):
    u"""Shipping events are things like 'OrderPlaced', 'Acknowledged', 'Dispatched', 'Refunded'"""
    # Code is used in forms
    code = models.CharField(max_length=128)
    # Name is the friendly description of an event
    name = models.CharField(max_length=255)
    # Code is used in forms
    code = models.SlugField(max_length=128)
    is_required = models.BooleanField(default=True, help_text="This event must be passed before the next shipping event can take place")
    # The normal order in which these shipping events take place
    sequence_number = models.PositiveIntegerField(default=0)
    
    def save(self, *args, **kwargs):
        if not self.code:
            self.code = slugify(self.name)
        super(AbstractShippingEventType, self).save(*args, **kwargs)
    
    class Meta:
        abstract = True
        verbose_name_plural = _("Shipping event types")
        ordering = ('sequence_number',)
        
    def __unicode__(self):
        return self.name
        
        
class AbstractOrderDiscount(models.Model):
    """
    A discount against an order.
    
    Normally only used for display purposes so an order can be listed with discounts displayed
    separately even though in reality, the discounts are applied at the line level.
    """
    order = models.ForeignKey('order.Order', related_name="discounts")
    offer = models.ForeignKey('offer.ConditionalOffer', null=True, on_delete=models.SET_NULL)
    voucher = models.ForeignKey('voucher.Voucher', related_name="discount_vouchers", null=True, on_delete=models.SET_NULL)
    voucher_code = models.CharField(_("Code"), max_length=128, db_index=True, null=True)
    amount = models.DecimalField(decimal_places=2, max_digits=12, default=0)
    
    class Meta:
        abstract = True
        
    def __unicode__(self):
        return u"Discount of %r from order %s" % (self.amount, self.order)    
        
    def description(self):
        if self.voucher_code:
            return self.voucher_code
        return self.offer.name



from django import forms
from django.utils.translation import ugettext_lazy as _
from django.forms.widgets import Input
from haystack.forms import FacetedSearchForm


class SearchInput(Input):
    '''
    Defining a search type widget

    This is an HTML5 thing and works nicely with Safari, other browsers default
    back to using the default "text" type
    '''
    input_type = 'search'


class MultiFacetedSearchForm(FacetedSearchForm):
    '''
    An extension of the regular faceted search form to alow for multiple facets
    '''
    q = forms.CharField(required=False, label=_('Search'), widget=SearchInput({"placeholder": _('Search')}))

    def search(self):
        '''
        Overriding the search method to allow for multiple facets
        '''
        sqs = super(FacetedSearchForm, self).search().order_by('-score')
        if hasattr(self, 'cleaned_data') and 'selected_facets' in self.cleaned_data:
            for f in self.cleaned_data['selected_facets'].split("|"):
                sqs = sqs.narrow(f)
        return sqs


from django.contrib import admin

from oscar.core.loading import import_module
import_module('offer.models', ['ConditionalOffer', 'Condition', 'Benefit', 'Range'], locals())


class ConditionAdmin(admin.ModelAdmin):
    list_display = ('type', 'value', 'range')


class BenefitAdmin(admin.ModelAdmin):
    list_display = ('__unicode__', 'type', 'value', 'range')
    
    
class ConditionalOfferAdmin(admin.ModelAdmin):
    list_display = ('name', 'offer_type', 'start_date', 'end_date', 'condition', 'benefit', 'total_discount')
    list_filter = ('offer_type',)
    readonly_fields = ('total_discount',)
    fieldsets = (
        (None, {
            'fields': ('name', 'description', 'offer_type', 'condition', 'benefit', 'start_date', 'end_date', 'priority')
        }),
        ('Usage', {
            'fields': ('total_discount',)
        }),
    )


admin.site.register(ConditionalOffer, ConditionalOfferAdmin)
admin.site.register(Condition, ConditionAdmin)
admin.site.register(Benefit, BenefitAdmin)
admin.site.register(Range)



from django.db import models
from django.utils.translation import ugettext_lazy as _
from django.template import Template, Context, TemplateDoesNotExist
from django.template.loader import get_template

from oscar.apps.customer.managers import CommunicationTypeManager


class AbstractEmail(models.Model):
    """
    This is a record of all emails sent to a customer.
    Normally, we only record order-related emails.
    """
    user = models.ForeignKey('auth.User', related_name='emails')
    subject = models.TextField(_('Subject'), max_length=255)
    body_text = models.TextField()
    body_html = models.TextField(blank=True, null=True)
    date_sent = models.DateTimeField(auto_now_add=True)

    class Meta:
        abstract = True

    def __unicode__(self):
        return u"Email to %s with subject '%s'" % (self.user.username, self.subject)


class AbstractCommunicationEventType(models.Model):
    
    # Code used for looking up this event programmatically.
    # eg. PASSWORD_RESET
    code = models.SlugField(max_length=128)
    
    # Name is the friendly description of an event for use in the admin
    name = models.CharField(max_length=255)
    
    # We allow communication types to be categorised
    ORDER_RELATED = 'Order related'
    USER_RELATED = 'User related'
    category = models.CharField(max_length=255, default=ORDER_RELATED)
    
    # Template content for emails
    email_subject_template = models.CharField(max_length=255, blank=True)
    email_body_template = models.TextField(blank=True, null=True)
    email_body_html_template = models.TextField(blank=True, null=True, help_text="HTML template")
    
    # Template content for SMS messages
    sms_template = models.CharField(max_length=170, blank=True, help_text="SMS template")
    
    date_created = models.DateTimeField(auto_now_add=True)
    date_updated = models.DateTimeField(auto_now=True)
    
    objects = CommunicationTypeManager()
    
    # File templates
    email_subject_template_file = 'customer/emails/commtype_%s_subject.txt'
    email_body_template_file = 'customer/emails/commtype_%s_body.txt'
    email_body_html_template_file = 'customer/emails/commtype_%s_body.html'
    sms_template_file = 'customer/sms/commtype_%s_body.txt'
    
    class Meta:
        abstract = True
        verbose_name_plural = _("Communication event types")

    def get_messages(self, ctx=None):
        """
        Return a dict of templates with the context merged in

        We look first at the field templates but fail over to 
        a set of file templates.  
        """
        if ctx is None:
            ctx = {}
        code = self.code.lower()
        # Build a dict of message name to Template instance
        templates = {'subject': 'email_subject_template',
                     'body': 'email_body_template',
                     'html': 'email_body_html_template',
                     'sms': 'sms_template'}
        for name, attr_name in templates.items():
            field = getattr(self, attr_name, None)
            if field:
                templates[name] = Template(field)
            else:
                template_name = getattr(self, "%s_file" % attr_name) % code
                try:
                    templates[name] = get_template(template_name)
                except TemplateDoesNotExist:
                    templates[name] = None
        
        messages = {}
        for name, template in templates.items():
            messages[name] = template.render(Context(ctx)) if template else ''

        # Ensure the email subject doesn't contain any newlines
        messages['subject'] = messages['subject'].replace("\n", "")

        return messages
        
    def __unicode__(self):
        return self.name   
    
    def is_order_related(self):
        return self.category == self.ORDER_RELATED
    
    def is_user_related(self):
        return self.category == self.USER_RELATED 
    


import csv

from oscar.core.loading import import_module
report_classes = import_module('reports.reports', ['ReportGenerator'])
analytics_models = import_module('analytics.models', ['ProductRecord', 'UserRecord'])


class ProductReportGenerator(report_classes.ReportGenerator):
    
    filename_template = 'product-analytics.csv'
    code = 'product_analytics'
    description = 'Product analytics'
    
    def generate(self, response):
        writer = csv.writer(response)
        header_row = ['Product',
                      'Views',
                      'Basket additions',
                      'Purchases',]
        writer.writerow(header_row)
        
        records = analytics_models.ProductRecord._default_manager.all()
        for record in records:
            row = [record.product, record.num_views, record.num_basket_additions, record.num_purchases]
            writer.writerow(row)
            
    def is_available_to(self, user):
        return user.is_staff
    
    def filename(self):
        return self.filename_template
    

class UserReportGenerator(report_classes.ReportGenerator):
    
    filename_template = 'user-analytics.csv'
    code = 'user_analytics'
    description = 'User analytics'
    
    def generate(self, response):
        writer = csv.writer(response)
        header_row = ['Username',
                      'Name',
                      'Date registered',
                      'Product views',
                      'Basket additions',
                      'Orders',
                      'Order lines',
                      'Order items',
                      'Total spent',
                      'Date of last order',
                      ]
        writer.writerow(header_row)
        
        records = analytics_models.UserRecord._default_manager.select_related().all()
        for record in records:
            row = [record.user.username, record.user.get_full_name(), record.user.date_joined, 
                   record.num_product_views, record.num_basket_additions, record.num_orders,
                   record.num_order_lines, record.num_order_items, record.total_spent, record.date_last_order]
            writer.writerow(row)
            
    def is_available_to(self, user):
        return user.is_staff
    
    def filename(self):
        return self.filename_template